#include <stdio.h>
#include <math.h>
#include <sys/time.h>


/*
#define TESTING
*/

#ifdef TESTING
static int numoftrials = 1; /*300*/;
#else
static int numoftrials = 1000; /*300*/;
#endif

#ifdef TESTING
#define PRECISION 4
#else
#define PRECISION 256
#endif
#define MULT_TABLE_MAX 16  /*if > 16, must change table*/
#define COARSENESS 0x8000
#define COARSENESS_SHIFTED 0x80000000
typedef struct {
  int lower;
  int upper;
} tempevent_type;

unsigned char mult_table[MULT_TABLE_MAX][MULT_TABLE_MAX];

typedef struct {
  int left[PRECISION+2];
  int right[PRECISION+2];
} tempdist_type;

typedef enum {FALSE, TRUE} bool;

tempdist_type uniform;
unsigned int uniform_tree_height = 0;

int maxmaxqueue = 0;
int maxqueue = 0;

int pregammas[101];

typedef struct {
  int value;
  unsigned int leftrods;
  unsigned int rightrods;
  void *left;
  void *right;
} tree_node_type;

tree_node_type spare_nodes[10000];
int allocated = 0;
tree_node_type *uniform_tree;

typedef struct {
  unsigned int aprecisionlower;
  unsigned int aprecisionupper;
  unsigned int arightremoved;
  unsigned int aleftremoved;
  unsigned int brightremoved;
  unsigned int bleftremoved;
  tree_node_type *arodstree;
  unsigned int agranulesmid;
  unsigned int agranulesrange;
} queue_node_type;

queue_node_type queue[PRECISION*8 + 2];

tree_node_type *build_spike_tree(unsigned int height) {
  tree_node_type *node, *finger, *build_uniform_tree();

  node = &spare_nodes[allocated++];
  
  if (height < 8)  {
    node->value = 0;
    node->left = NULL;
    node->leftrods = 0;
    node->rightrods = PRECISION;
    node->right = build_spike_tree(height + 1);
    }
  else {
    node->value = 0;
    node->left = NULL;
    node->right = build_uniform_tree(PRECISION,0,0,PRECISION);
    }
  return node;
}

tree_node_type *build_uniform_tree(unsigned int current, unsigned int height,
				   unsigned int lower, unsigned int upper) {
  tree_node_type *node, *finger;

  node = &spare_nodes[allocated++];
  node->value = lower + ((upper - lower) >> 1);
  if (current == 1) {
    node->leftrods = node->value;
    node->rightrods = PRECISION - node->value - 1;
    /*
    if (upper == PRECISION) node->value = PRECISION;
    if (upper == PRECISION) node->leftrods = PRECISION;
    if (upper == PRECISION) node->rightrods = 0;
    */
#ifdef TESTING
  printf("node %d leftrods %d rightrods %d\n", node->value, node->leftrods, node->rightrods);
#endif
    node->left = NULL;
    node->right = NULL;
    }
  else {
    node->leftrods = node->value;
    node->rightrods = PRECISION - node->value;
#ifdef TESTING
  printf("node %d leftrods %d rightrods %d\n", node->value, node->leftrods, node->rightrods);
#endif
    node->left = (void *)build_uniform_tree(current >> 1,height+1,lower,node->value);
    node->right = (void *)build_uniform_tree(current >> 1,height+1,node->value,upper);
    }
  if (height > uniform_tree_height) uniform_tree_height = height;
  /*
  node->height = height;
  */
  return node;
}


int main(argc, argv) 
int argc;
char *argv[]; {
  tree_node_type *build_uniform_tree();
  int i, j, k, sum;
  int times[101];
  int temps[101];
  void gettimeofday();
  bool before();
  struct timeval tp1, tp2;
  int temp1, temp2;
  int gamma, temp;
  tempevent_type alpha, beta;
  double f;

  temp = 0;
  alpha.lower = 1;
  alpha.upper = 8600;
  beta.lower = 1;
  beta.upper = 8600; 
  if (argc > 3) { 
    alpha.lower = atoi(argv[1]);
    alpha.upper = atoi(argv[2]);
    beta.lower = atoi(argv[3]);
    beta.upper = atoi(argv[4]);
    }

  for (i=0;i<MULT_TABLE_MAX;i++) for (j=0;j<MULT_TABLE_MAX;j++) mult_table[i][j] = i*j;

#define GAMMA_MULT PRECISION * PRECISION / 100.0
  for (i=0;i<101;i++) {
    f = i;
    pregammas[i] = (int) ceil(f * GAMMA_MULT);
    /* printf("%d %d %f \n", i, pregammas[i], (i * (GAMMA_MULT)));*/
    }

  uniform_tree = build_uniform_tree(PRECISION,0,0,PRECISION);
  /*
  uniform_tree = build_spike_tree(1);
  */

  for (i=1+numoftrials;i<=100+numoftrials;i++) {

    gettimeofday(&tp1,NULL);

    for (j=0; j<numoftrials; j++) temp += before(i - numoftrials,&alpha,&beta);

    gettimeofday(&tp2,NULL);

    temp1 = tp1.tv_sec * 1000000 + tp1.tv_usec;
    temp2 = tp2.tv_sec * 1000000 + tp2.tv_usec;
    temp1 = temp2 - temp1;

    times[i-numoftrials] = temp1;
    temps[i-numoftrials] = temp;
    }
    for (sum=0,i = 1; i <= 100; i++) {
      sum += times[i];
#ifdef GRAPHING
      printf("%d\n", times[i]/numoftrials);
#else
      printf("%d %d %d\n", temps[i], i, times[i]/numoftrials);
#endif
      }

#ifndef GRAPHING
   printf("Average %d\n", sum/numoftrials);
   printf("m %d\n", maxmaxqueue);
#endif

return FALSE;
}

#include "shared.c"
