/***********************************************************************\ 
*                                                                       * 
*   File: calendric_system/newh/calendar/gregorian/gregorian.h
*                                                                       * 
*   The MultiCal System is free software in the public domain; you can  * 
*   redistribute it and/or modify it as you wish. We ask that you       * 
*   retain credits referencing the University of Arizona and that you   * 
*   identify any changes you make.                                      * 
*                                                                       * 
*   Report problems to multical@cs.arizona.edu                          * 
*   Direct all inquiries to:    The MultiCal Project                    * 
*                               Department of Computer Science          * 
*                               University of Arizona                   * 
*                               Tucson, AZ 85721                        * 
*                               U.S.A.                                  * 
*                                                                       * 
\***********************************************************************/ 

#ifndef gregorian
#define gregorian

#include "gregorian_comparison.h"
#include "calendar.h"

int printf();
int fprintf();
int fflush();

/* gregorian.h
*
*  This module contains all basic information used by the gregorian
*  calendar system.  
*
*/

typedef enum { gregorian_month, gregorian_year} gregorian_span_type;

typedef enum { gregorian_eq, gregorian_lt, gregorian_gt } gregorian_comparison_type;

typedef struct {
  unsigned long long  year;
  unsigned long month;
  unsigned long day;
  unsigned long hour;
  unsigned long minute;
  unsigned long second;
  unsigned long milli;
  unsigned long micro;
  unsigned long nano;
} greg_space_type;

typedef greg_space_type *greg_type;


typedef struct {
    short a;
    short b;
} signed_span;

typedef struct {
    unsigned a : 16;
    unsigned b : 16;
} unsigned_span;

typedef union { 
    signed_span s;
    unsigned_span u;
    unsigned int  i;
} gregorian_span_length_type;


/*
* Note that 12 month spans = 1 year spans, but
*   12 Gregorian Months(in seconds) != 1 Gregorian Year(in seconds)
*/

#define Gregorian_Day_Seconds   24*60*60
#define Gregorian_Month_Seconds 31*Gregorian_Day_Seconds /*secs in month span*/
#define Gregorian_Year_Seconds  365*Gregorian_Day_Seconds /*secs in year span*/

#define Gregorian_Year_Months   12
#define Gregorian_Month_Days    31

/* 
* Next follow some Gregorian specific functions 
*/

void gregorian_calendar_init(unsigned char cal_id, calendar_type **);
error_type gregorian_create_month(actual_arg_type *args, actual_arg_type*);
error_type gregorian_create_year(actual_arg_type *args, actual_arg_type *);

void gregorian_internal_error(char*s);
error_type gregorian_internal_create_vs(short a, short b, gregorian_span_type kind, span_type spanin);

error_type gregorian_vs_compare_vs(span_type vs1, span_type vs2, gregorian_comparison_type *result);
error_type gregorian_vs_compare_fs(span_type vs, span_type fs, gregorian_comparison_type *result);

error_type gregorian_date_to_secs( greg_type, seconds_type);
error_type gregorian_secs_to_date( seconds_type, greg_type);
error_type gregorian_add_vs_event(vs_space_type vs, greg_type e, event_type r);
void inputfull(greg_type gdate, long long *); 
void outputfull(long long *, greg_type gdate);

/*
 * global gregorian variables
 */

#define Num_Gregorian_Functions 2

extern unsigned char greg_cal_id;
extern seconds_type greg_secs1, greg_secs2;
extern span_type greg_s1, greg_s2;
extern event_type greg_e1, greg_e2, greg_e3;
extern span_type greg_1_chronon;
extern function_entry_type gregorian_functions[];
extern int greg_month_table[];

/* other gregorian standards */
#define LEAP400 97
#define SECONDS_IN_HOUR 3600
#define SECONDS_IN_MINUTE 60
#define SECONDS_IN_DAY 86400
#define SECONDS_IN_YEAR 86400*365
#define SECONDS_IN_LEAP_YEAR 86400*366
#define LEAPDAYS_UPTO_1970 477
#define YEARS_IN_GREG 400
#define DAYS_IN_GREG 146097
#define SECONDS_IN_GREG 0x2f0605980ull
#define SECONDS_TO_1970 0xe7791f700ull

/*
#define SECONDS_IN_GREG (DAYS_IN_GREG * SECONDS_IN_DAY)
#define SECONDS_TO_1970 ((621 * 100000000) + 67132800)
*/

#define century_year(x)  (x % 100 == 0)
#define year_div_by_400(x) (x % 400 == 0)
#define leap_year(x) ((x % 4 == 0) && ( year_div_by_400(x) || (!century_year(x))))


/* external variable references */

/* inputfull */
extern unsigned long day_to_seconds[];
extern unsigned long month_to_seconds[];
extern unsigned long leap_month_to_seconds[];
extern unsigned long long year_to_seconds[];
extern unsigned char leap_year[];
extern unsigned long yr[];
extern unsigned long long sg[];
extern unsigned long long years_to_gregs_nibbles[9][16];
extern unsigned long years_to_gregs_residue_nibbles[9][16];

/* outputfull */
extern unsigned int year_count[];
extern unsigned int year_residue[];
extern unsigned int sr[];
extern unsigned int dr[];
extern unsigned char leap_year[];
extern unsigned long day_to_day[];
extern unsigned long day_to_leap_day[];
extern unsigned long day_to_month[];
extern unsigned long day_to_leap_month[];
extern unsigned int dy[];
extern unsigned long greg_nibbles[15][16];
extern unsigned long day_nibbles[15][16];
unsigned long second_nibbles[15][16];

#endif
