/***********************************************************************\ 
*                                                                       * 
*   File: calendric_system/newh/tadt_internal/int.h
*                                                                       * 
*   The MultiCal System is free software in the public domain; you can  * 
*   redistribute it and/or modify it as you wish. We ask that you       * 
*   retain credits referencing the University of Arizona and that you   * 
*   identify any changes you make.                                      * 
*                                                                       * 
*   Report problems to multical@@cs.arizona.edu                          * 
*   Direct all inquiries to:    The MultiCal Project                    * 
*                               Department of Computer Science          * 
*                               University of Arizona                   * 
*                               Tucson, AZ 85721                        * 
*                               U.S.A.                                  * 
*                                                                       * 
\***********************************************************************/ 
#ifndef tadt_int
#define tadt_int

#define MAX_PRECISION 30
#define MAX_SCALE 30

#include "intops.h"

typedef union {
  int32  small;
  int64  medium;
  int96  big;
} polymorphic_int;

typedef enum {SIZE32, SIZE64, SIZE96} size_type;

/*
*  STANDARD EVENT TIMESTAMPS:
*/

#define DETERMINATE_MASK 0xF0000000
typedef struct {
  unsigned ts_type :  3;   /* determinate                        */
  unsigned sign    :  1;   /* sign flag -- 0 positive 1 negative */
  unsigned data    : 28;   /* data                               */
} determinate_type;

#define NOWRELATIVE_MASK 0xF0000000
typedef struct {
  unsigned ts_type :  3;   /* nowrelative                        */
  unsigned sign    :  1;   /* sign flag -- 0 positive 1 negative */
  unsigned data    : 28;   /* data                               */
} nowrelative_type;

typedef struct {
  unsigned ts_type :  3;   /* special                            */
  unsigned vspan   :  1;   /*   ---- variable span flag ---      */
			   /* the variable span format looks like*/
			   /* a determinate event, remainder of  */
			   /* this format is the special format  */
  unsigned which   : 28;   /* which special event                */ 
                           /* word boundary                      */
} special_type;

typedef struct {
  unsigned ts_type :  16;   /* upack                              */
  unsigned base_ts_type:3; /* underlying timestamp type          */
  unsigned modified:  1;   /* modified or can repack?            */
  unsigned wasted  : 12;   /* wasted                             */
} upack_determinate_type;

typedef struct {
  upack_determinate_type wasted;   /* wasted space */
  int32                  granules; /* 32 bit granules */ 
} upack32_determinate_type;

typedef struct {
  upack_determinate_type wasted;   /* wasted space */
  int64                  granules; /* 64 bit granules */ 
} upack64_determinate_type;

typedef struct {
  upack_determinate_type wasted;   /* wasted space */
  int96                  granules; /* 96 bit granules */ 
} upack96_determinate_type;

#ifdef INDETERMINATE
/*
*  CHUNKED INDETERMINATE FORM
*/

#define CHUNKED_STANDARD_MASK 0xFFFF0000
typedef struct {
  unsigned ts_type :  3;   /* chunked_standard                   */
  unsigned missing :  1;   /* missing or uniform                 */
  unsigned chunksize: 4;   /* size of chunk                      */
  unsigned chunks  :  7;   /* number of chunks                   */
  unsigned sign    :  1;   /* sign flag -- 1 positive 0 negative */
  unsigned data    : 16;   /* granules                           */
} chunked_standard_type;

/*
*  NONSTANDARD, CHUNKED INDETERMINATE FORM
*/

#define CHUNKED_NONSTANDARD_MASK 0xE0000000
typedef struct {
  unsigned ts_type :  3;   /* chunked_nonstandard                */
  unsigned chunksize: 4;   /* size of chunk                      */
  unsigned chunks  :  7;   /* number of chunks                   */
  unsigned right_off: 7;   /* right offset                       */
  unsigned left_off:  7;   /* left offset                        */
  unsigned disthi  :  4;   /* normalized distribution            */
                           /* word boundary                      */
  unsigned distlow :  3;   /* normalized distribution            */
  unsigned sign    :  1;   /* sign flag -- 1 positive 0 negative */
  unsigned data    : 28;   /* data field                         */
} chunked_nonstandard_type;

/*
*  GENERAL, STANDARD INDETERMINATE FORMS
*/

typedef struct {
  unsigned ts_type :  3;   /* general_standard                   */
  unsigned sign    :  1;   /* sign flag -- 1 positive 0 negative */
  unsigned data    : 28;   /* data                               */
                           /* word boundary                      */
  unsigned upper_special:1;/* the upper instant is a spcial ins. */
  unsigned lower_special:1;/* the lower instant is a spcial ins. */
  unsigned missing :  1;   /* the missing or uniform distribution*/
  unsigned otherdata:29;   /* interpreted as determinate instant */
} general32_standard_type;

typedef struct {
  unsigned ts_type :  3;   /* general_standard                   */
  unsigned sign    :  1;   /* sign flag -- 1 positive 0 negative */
  unsigned data    : 28;   /* data                               */
                           /* word boundary                      */
  unsigned data2   : 32;   /* Interpreted as determinate instant */
                           /* word boundary                      */
  unsigned upper_special:1;/* the upper instant is a spcial ins. */
  unsigned lower_special:1;/* the lower instant is a spcial ins. */
  unsigned missing :  1;   /* the missing or uniform distribution*/
  unsigned otherdata:29;   /* interpreted as determinate instant */
                           /* word boundary                      */
  unsigned other2  : 32;   /* interpreted as determinate instant */
} general64_standard_type;

typedef struct {
  unsigned ts_type :  3;   /* general_standard                   */
  unsigned sign    :  1;   /* sign flag -- 1 positive 0 negative */
  unsigned data    : 28;   /* data                               */
                           /* word boundary                      */
  unsigned data2   : 32;   /* Interpreted as determinate instant */
                           /* word boundary                      */
  unsigned data3   : 32;   /* Interpreted as determinate instant */
                           /* word boundary                      */
  unsigned upper_special:1;/* the upper instant is a spcial ins. */
  unsigned lower_special:1;/* the lower instant is a spcial ins. */
  unsigned missing :  1;   /* the missing or uniform distribution*/
  unsigned otherdata:29;   /* interpreted as determinate instant */
                           /* word boundary                      */
  unsigned other2  : 32;   /* interpreted as determinate instant */
                           /* word boundary                      */
  unsigned other3  : 32;   /* interpreted as determinate instant */
} general96_standard_type;

/*
*  GENERAL, NONSTANDARD INDETERMINATE FORMS
*/

typedef struct {
  unsigned ts_type :  3;   /* general_nonstandard                */
  unsigned sign    :  1;   /* sign flag -- 1 positive 0 negative */
  unsigned data    : 28;   /* data                               */
                           /* word boundary                      */
  unsigned wasted  :  7;   /* wasted                             */
  unsigned right_off: 7;   /* right offset                       */
  unsigned left_off:  7;   /* left offset                        */
  unsigned dist    :  7;   /* normalized distribution            */
                           /* word boundary                      */
  unsigned upper_special:1;/* the upper instant is a spcial ins. */
  unsigned lower_special:1;/* the lower instant is a spcial ins. */
  unsigned wasted2 :  1;   /* wasted                             */
  unsigned otherdata:29;   /* interpreted as determinate instant */
} general32_nonstandard_type;

typedef struct {
  unsigned ts_type :  3;   /* general_nonstandard                */
  unsigned sign    :  1;   /* sign flag -- 1 positive 0 negative */
  unsigned data    : 28;   /* data                               */
                           /* word boundary                      */
  unsigned data2   : 32;   /* Interpreted as determinate instant */
                           /* word boundary                      */
  unsigned wasted  :  7;   /* wasted                             */
  unsigned right_off: 7;   /* right offset                       */
  unsigned left_off:  7;   /* left offset                        */
  unsigned dist    :  7;   /* normalized distribution            */
                           /* word boundary                      */
  unsigned upper_special:1;/* the upper instant is a spcial ins. */
  unsigned lower_special:1;/* the lower instant is a spcial ins. */
  unsigned wasted2 :  1;   /* wasted                             */
  unsigned otherdata:29;   /* interpreted as determinate instant */
                           /* word boundary                      */
  unsigned other2  : 32;   /* interpreted as determinate instant */
} general64_nonstandard_type;

typedef struct {
  unsigned ts_type :  3;   /* general_nonstandard                */
  unsigned sign    :  1;   /* sign flag -- 1 positive 0 negative */
  unsigned data    : 28;   /* data                               */
                           /* word boundary                      */
  unsigned data2   : 32;   /* Interpreted as determinate instant */
                           /* word boundary                      */
  unsigned data3   : 32;   /* Interpreted as determinate instant */
                           /* word boundary                      */
  unsigned wasted  :  7;   /* wasted                             */
  unsigned right_off: 7;   /* right offset                       */
  unsigned left_off:  7;   /* left offset                        */
  unsigned dist    :  7;   /* normalized distribution            */
                           /* word boundary                      */
  unsigned upper_special:1;/* the upper instant is a spcial ins. */
  unsigned lower_special:1;/* the lower instant is a spcial ins. */
  unsigned wasted2 :  1;   /* wasted                             */
  unsigned otherdata:29;   /* interpreted as determinate instant */
                           /* word boundary                      */
  unsigned other2  : 32;   /* interpreted as determinate instant */
                           /* word boundary                      */
  unsigned other3  : 32;   /* interpreted as determinate instant */
} general96_nonstandard_type;

/*
*  Probability Distribution Function Format
*/

typedef struct {
  unsigned wasted  : 10;   /* wasted                             */
  unsigned name    :  8;   /* normalized distribution            */
  unsigned right_off: 7;   /* right offset                       */
  unsigned left_off:  7;   /* left offset                        */
} prob_dist_type;

#define SIG_BITS 16
#define MAX_SC 65536 /* This is 2**SIG_BITS */
#define MIN_SC    0
#define GAMMA_MULT 655 /* This is (DIST_MAX+1**2)/100 */
#define DIST_MIN 0
#define DIST_MAX 255
#define DEFINITE 100
#define POSSIBLE 1
#define HOW_MANY_CHUNKS 128
#define MAX_CHUNK 16
#define DEFAULT_DISTRIBUTION 0
#define HOW_MANY_DISTS   64 
#define UNIFORM     0

typedef struct {
  unsigned short ceiling[DIST_MAX+1]; /* sampled distribution values */
  unsigned short floor[DIST_MAX+1]; /* sampled distribution values */
} dist_space_type;

typedef dist_space_type *dist_type;

/* For the distributions name table */
#define UNIFORM           0
#define MISSING           1
#define MAX_DISTRIBUTION  2

/*
typedef struct {
  long long start;
  long long finish;
  unsigned long long range;
  unsigned short      tag;
  unsigned short      left_offset;
  unsigned short      right_offset;
  unsigned short      from_sc;
  unsigned short      to_sc;
  dist_type           dist;
} ind_space_type;

typedef ind_space_type *ind_type;
*/

typedef struct {
  unsigned int        from;
  unsigned int        to;
  unsigned short      from_prob;
  unsigned short      to_prob;
  unsigned short      prob;
} node_sub_space_type;

typedef struct {
  node_sub_space_type alpha;
  node_sub_space_type beta;
  int                next;
} node_space_type;

typedef node_space_type *node_type;

typedef struct {
  int mid_prob;
  int prob_left;
  int prob_right;
} event_aux_space_type;

/* Pad this sucker out to a word boundary for sure */
typedef struct {
  unsigned ts_type :  16;   /* upack                              */
  unsigned base_ts_type:3; /* underlying timestamp type          */
  unsigned modified:  1;   /* modified or can repack?            */
  unsigned wasted  : 12;   /* wasted                             */
  prob_dist_type distribution;  /* distribution info                  */
  upack_determinate_type lower; /* lower support                      */
  upack96_determinate_type lower_space;  /* lower support */
  upack_determinate_type upper;  /* upper support */
  upack96_determinate_type upper_space; /* upper support */
} upack_indeterminate_type;

#endif

/*
*  VARIABLE SPAN FORMAT
*/

typedef struct {
  unsigned wasted  :  4;   /* see special format                 */
  unsigned cal_sys :  6;   /* which calendric system             */
  unsigned dataone : 22;   /* calendar interpreted data          */ 
                           /* word boundary                      */
  unsigned datatwo : 32;   /* calendar interpreted data          */ 
} vspan_type;

/*
*  TIME-STAMP STORAGE STRUCTURES
*/ 

typedef union {
  determinate_type   generic;        /* For ts_type access */
  determinate_type   determinate;
  special_type       special;
  upack_determinate_type      upack;
  upack32_determinate_type   upack32;
  upack64_determinate_type   upack64;
  upack96_determinate_type   upack96;

#ifdef NOWRELATIVE
  nowrelative_type   nowrelative;
#endif 

#ifndef INDETERMINATE
  unsigned int                word[sizeof(upack_determinate_type)/4];
  unsigned short              sword[sizeof(upack_determinate_type)/2];
#else            
  unsigned int                word[(sizeof(upack_indeterminate_type)*2)/4];
  unsigned short              sword[(sizeof(upack_indeterminate_type)*2)/4];

  upack_indeterminate_type    upack_ind;
  chunked_standard_type       chunked;
  chunked_nonstandard_type    chunked_nonstandard;
  general32_standard_type     general32;
  general64_standard_type     general64;
  general96_standard_type     general96;
  general32_nonstandard_type  general32_nonstandard;
  general64_nonstandard_type  general64_nonstandard;
  general96_nonstandard_type  general96_nonstandard;
#ifdef INDETERMINATE_NOWRELATIVE
  indeterminate_nowrelative_type indeterminate_nowrelative;
#endif
#endif

} event_space_type;

typedef union {
  determinate_type           generic;        /* For ts_type access */
  vspan_type                 vspan;
  determinate_type           determinate;
  special_type               special;
  upack_determinate_type     upack;
  upack32_determinate_type   upack32;
  upack64_determinate_type   upack64;
  upack96_determinate_type   upack96;

#ifdef NOWRELATIVE
  nowrelative_type   nowrelative;
#endif 

#ifndef INDETERMINATE
  unsigned int                word[sizeof(upack_determinate_type)/4];
  unsigned short              sword[sizeof(upack_determinate_type)/2];
#else            
  unsigned int                word[(sizeof(upack_indeterminate_type)*2)/4];
  unsigned short              sword[(sizeof(upack_indeterminate_type)*2)/4];

  upack_indeterminate_type    upack_ind;
  chunked_standard_type       chunked;
  chunked_nonstandard_type    chunked_nonstandard;
  general32_standard_type     general32;
  general64_standard_type     general64;
  general96_standard_type     general96;
  general32_nonstandard_type  general32_nonstandard;
  general64_nonstandard_type  general64_nonstandard;
  general96_nonstandard_type  general96_nonstandard;
#endif

} span_space_type;

typedef struct {
  event_space_type *start;
  event_space_type *terminate;
} interval_space_type;

typedef union {
  span_space_type      span;
  event_space_type     event;
  interval_space_type  interval;
} timestamp_space_type;

#define MAX_EVENT_SIZE (sizeof(event_space_type)/4)
#define MAX_INTERVAL_SIZE (sizeof(interval_space_type)/4)
#define MAX_SPAN_SIZE (sizeof(span_space_type)/4)

#endif
