/***********************************************************************\ 
*                                                                       * 
*   File: calendric_system/newh/ucs/output.h
*                                                                       * 
*   The MultiCal System is free software in the public domain; you can  * 
*   redistribute it and/or modify it as you wish. We ask that you       * 
*   retain credits referencing the University of Arizona and that you   * 
*   identify any changes you make.                                      * 
*                                                                       * 
*   Report problems to multical@cs.arizona.edu                          * 
*   Direct all inquiries to:    The MultiCal Project                    * 
*                               Department of Computer Science          * 
*                               University of Arizona                   * 
*                               Tucson, AZ 85721                        * 
*                               U.S.A.                                  * 
*                                                                       * 
\***********************************************************************/ 
#ifndef ucs_output
#define ucs_output
/* output.h
*
*  This module implements UCS operations for 
*  time-stamp to temporal constant conversion.
*
*/

/*  
*  Routine:  ucs_e_to_string
*
*  Description:  Convert a timestamp to an event string according to the
*		 the specification of the Event Output Format,
*                Locale, Beginning, and Forever properties
*
*  Arguments: event -- (IN) :  the timestamp to be converted 
*             str   -- (OUT):  the string to be returned
*
*  Return Value: Error code
*
*  Errors: conversion_error
*          string_overflow
*  
*  Side Effects: Overwrites str
*/
error_type ucs_e_to_string(event_type event, string_struct str);

/*  
*  Routine:  ucs_i_to_string
*
*  Description:  Convert a timestamp to an interval string according to the
*		 specification of the Interval Output Format and
*                Locale properties
*
*  Arguments: interval -- (IN) :  the timestamp to be converted 
*             str      -- (OUT):  the string to be returned
*
*  Return Value: Error code
*
*  Errors: conversion_error
*          string_overflow
*  
*  Side Effects: Overwrites str
*/
error_type ucs_i_to_string(interval_type interval, string_struct str);

/*  
*  Routine:  ucs_s_to_string
*
*  Description:  Convert a timestamp to a span string according to the
*		 specification of the Span Output Format property
*
*  Arguments: span --  (IN) : the timestamp to be converted 
*             str  --  (OUT): the string to be returned
*
*  Return Value: Error code
*
*  Errors: conversion_error
*          string_overflow
*  
*  Side Effects: Overwrites str
*/
error_type ucs_s_to_string(span_type span, string_struct str);
#endif
