/***********************************************************************\ 
*                                                                       * 
*   File: calendric_system/tadt/test/arithmetic_test.c
*                                                                       * 
*   The MultiCal System is free software in the public domain; you can  * 
*   redistribute it and/or modify it as you wish. We ask that you       * 
*   retain credits referencing the University of Arizona and that you   * 
*   identify any changes you make.                                      * 
*                                                                       * 
*   Report problems to multical@cs.arizona.edu                          * 
*   Direct all inquiries to:    The MultiCal Project                    * 
*                               Department of Computer Science          * 
*                               University of Arizona                   * 
*                               Tucson, AZ 85721                        * 
*                               U.S.A.                                  * 
*                                                                       * 
\***********************************************************************/ 
/* Test code for the tadt arithmetic routines */
#include <stdio.h>
#include <stdlib.h>
#include "system_wide.h"
#include "tadt.h"
#define MAXEV 11

context_space_type context_space;
context_type context = &context_space;

void main()
{
  event_space_type ev[special][MAXEV];
  event_space_type upack[special][MAXEV];
  event_space_type result[special][MAXEV];
  event_space_type final[special][MAXEV];
  span_space_type span[special][MAXEV];
  seconds_space_type sec[3];
  int one, i, j;
  double temp;
  static void init_determinate(), dump_seconds();
  void printf();

  context->precision.precision = 5;

  error_handler();
  tadt_init();

  /*
  evsp1 = tadt_allocate_e(special);  
  evsp2 = tadt_allocate_e(special);  
  evsp3 = tadt_allocate_e(special);  
  */

  /* Events to test (one of each resolution) */
  /*                         e9 */
  /*                      e8    */
  /*                   e7       */
  /*                e6          */
  /*            origin          */
  /*          e4                */
  /*       e3                   */
  /*    e2                      */
  /* e1                         */

  init_determinate(&ev[determinate][0], -5);
  init_determinate(&ev[determinate][1], -4);
  init_determinate(&ev[determinate][2], -3);
  init_determinate(&ev[determinate][3], -2);
  init_determinate(&ev[determinate][4], -1);
  init_determinate(&ev[determinate][5], 0);
  init_determinate(&ev[determinate][6], 1);
  init_determinate(&ev[determinate][7], 2);
  init_determinate(&ev[determinate][8], 3);
  init_determinate(&ev[determinate][9], 4);
  init_determinate(&ev[determinate][10], 5);

  tadt_unpack_e(&ev[determinate][4], &upack[0][0]);
  tadt_e_to_seconds(&ev[determinate][4], &sec[0]);
  dump_seconds(&sec[0]);
  dump_event(&ev[determinate][4], "determinate guy 4.1" );
  tadt_seconds_to_e(&sec[0], &ev[determinate][4]);
  dump_event(&ev[determinate][4], "determinate guy 4.2") ;

  for (i=determinate; i<special; i++) for (j=0; j<MAXEV; j++) {
    tadt_unpack_e(&ev[i][j], &upack[i][j]);
    tadt_unpack_fs((span_type) &ev[i][j], &span[i][j]);
    }

  /* Division test */
  /* 
  printf("division test\n");
  tadt_fs_div_n(&span[hi][4], 2, &result[0][0]);
  dump_ev(&span[hi][4]);
  printf("answer is \n");
  dump_ev(&result[0][0]);
  tadt_fs_div_fs(&span[low][MAXEV-1], &span[hi][1], &temp);
  dump_ev(&span[low][MAXEV-1]);
  dump_ev(&span[hi][1]);
  printf("answer is %f\n", temp);
  */

  /* Addition test */
  printf("addition test\n");
  tadt_e_add_fs(&upack[determinate][4], &span[determinate][4], &result[0][0]);
  dump_event(&ev[determinate][4], "op1 + ");
  dump_event(&upack[determinate][4], "op 2 (unpacked)");
  dump_event(&span[determinate][4], "op 2 (packed)");
  dump_event(&result[0][0], "result");
  tadt_e_to_seconds(&result[0][0],&sec[0]);
  dump_seconds(&sec[0]);
  printf("minus test\n");
  tadt_e_minus_fs(&result[0][0], &span[determinate][4], &result[1][0]);
  dump_event(&result[0][0], "op1");
  dump_event(&span[determinate][4], "op2 packed");
  dump_event(&result[1][0], "result");
  tadt_e_to_seconds(&result[1][0],&sec[0]);
  dump_seconds(&sec[0]);
  printf("testing\n");
  
  /*
  for (i=hi; i<special; i++) for (j=0; j<MAXEV; j++) {
    if ((one = tadt_e_add_fs(&upack[i][j], &span[i][j], &result[i][j])) != tadt_ok) internal_error("%s %d", "out of range?", one);
    printf("i %d j %d\n", i, j);
    dump_ev(&ev[i][j]);
    dump_ev(&upack[i][j]);
    dump_ev(&span[i][j]);
    dump_ev(&result[i][j]);
    tadt_repack_e(&result[i][j],&final[i][j]);
    dump_ev(&final[i][j]);
    }
*/

  /* Negation test */
  /*
  for (i=hi; i<special; i++) for (j=0; j<MAXEV; j++) {
    if ((one = tadt_neg_fs(&span[i][j], &result[i][j])) != tadt_ok) {
       printf("\none %x\n", one);
       getchar();
       internal_error("%s", "notok");
       }
    printf("i %d j %d\n", i, j);
    dump_ev(&ev[i][j]);
    dump_ev(&span[i][j]);
    dump_ev(&result[i][j]);
    }
  */
  /* Multiplication test */
  /*
  for (i=hi; i<special; i++) for (j=0; j<MAXEV; j++) {
    if ((one = tadt_fs_times_n(&span[i][j], 1.5, &result[i][j])) != tadt_ok) {
       printf("\none %x\n", one);
       getchar();
       internal_error("%s", "notok");
       }
    printf("i %d j %d\n", i, j);
    dump_ev(&ev[i][j]);
    dump_ev(&span[i][j]);
    dump_ev(&result[i][j]);
    }
  */
}

static void init_determinate(event_type ev, int32 granules)
{
  ev->determinate.ts_type = determinate;
  ev->determinate.data = granules;
  ev->determinate.sign = (granules < 0)? 1 : 0;
}

static void dump_seconds(seconds_type s)
{
  printf("seconds %d %5d\n", s->seconds32, s->special);
}
